import { isEmpty, isDate, isObject, isFunction, isString, padStart, padEnd } from './utils';

var ensureDate = function ensureDate(date) {
	if (isDate(date)) return date;
	if (isObject(date)) {
		// for moment() like object
		if (isFunction(date.toDate)) return date.toDate();

		// for luxon.DateTime
		if (isFunction(date.toJSDate)) return date.toJSDate();
	}
	if (/^\d+$/.test(date)) return new Date(+padEnd(date, 13, '0'));
	return new Date(date);
};

var validNumber = function validNumber(val) {
	if (!/^-?\d+\.?\d*$/.test(val)) {
		throw new Error(val + ' is NOT a valid "number" type');
	}
	return true;
};

var toNumber = function toNumber(val) {
	return validNumber(val) && (+val || 0);
};
var toInt = function toInt(val) {
	return ~~toNumber(val);
};

var toStr = function toStr(val) {
	return val.toString();
};
var toBoolean = function toBoolean(val) {
	return !!val && val !== '0' && val !== 'false' && val !== 'undefined' && val !== 'null';
};

var toDateTime = function toDateTime(val) {
	// handle multi dateTime string by "," seperator
	if (isString(val) && ~val.indexOf(',')) {
		return val.split(',').map(toDateTime).join(',');
	}

	try {
		return toDate(val) + 'T' + toTime(val);
	} catch (err) {
		throw new Error('Can NOT convert "' + val + '" to "dateTime" format');
	}
};

var toDate = function toDate(val) {
	// handle multi date string by "," seperator
	if (isString(val) && ~val.indexOf(',')) {
		return val.split(',').map(toDate).join(',');
	}

	try {
		var d = ensureDate(val);
		if (d.toString() === 'Invalid Date') throw new Error();
		var year = d.getFullYear();
		var month = padStart(d.getMonth() + 1, 2, 0);
		var date = padStart(d.getDate(), 2, 0);
		return year + '-' + month + '-' + date;
	} catch (err) {
		throw new Error('Can NOT convert "' + val + '" to "date" format');
	}
};

var toTime = function toTime(val) {
	// handle multi time string by "," seperator
	if (isString(val) && ~val.indexOf(',')) {
		return val.split(',').map(toTime).join(',');
	}

	try {
		var d = ensureDate(val);
		if (d.toString() === 'Invalid Date') throw new Error();
		var hours = padStart(d.getHours(), 2, 0);
		var minutes = padStart(d.getMinutes(), 2, 0);
		var seconds = padStart(d.getSeconds(), 2, 0);
		var partialTime = hours + ':' + minutes + ':' + seconds;
		var zoneOffset = -d.getTimezoneOffset();

		/* istanbul ignore next */
		var offsetDif = zoneOffset < 0 ? '-' : '+';

		var offsetZoneHours = padStart(Math.abs(zoneOffset / 60), 2, 0);
		var offsetZoneMinutes = padStart(Math.abs(zoneOffset % 60), 2, 0);
		var offset = '' + offsetDif + offsetZoneHours + ':' + offsetZoneMinutes;
		return partialTime + offset;
	} catch (err) {
		throw new Error('Can NOT convert "' + val + '" to "time" format');
	}
};

export var formatHelper = {
	integer: toInt,
	number: toNumber,
	string: toStr,
	boolean: toBoolean,
	date: toDate,
	time: toTime,
	dateTime: toDateTime
};

export var FormatTypesKeys = Object.keys(formatHelper);

// TODO: add cache
export function createFormatFunc(type) {
	return function format(val) {
		if (isEmpty(val)) return val;
		return formatHelper[type](val);
	};
}